
export(accProb)
export(optAttrPlan)
export(optVarPlan)
export(optPlan)
export(muEst)
export(OCdata)
export(manualPlan)
S3method(as.list, OCdata)
S3method(as.data.frame, OCdata)
S3method(OCdata, AttrPlan)
S3method(OCdata, VarPlan)
S3method(accProb, AttrPlan)
S3method(accProb, VarPlan)
S3method(plot, AttrPlan)
S3method(plot, VarPlan)
S3method(summary, AttrPlan)
S3method(summary, VarPlan)
S3method(print, AttrPlan)
S3method(print, VarPlan)
S3method(print, OCdata)
S3method(summary, OCdata)
S3method(plot, OCdata)
# Importing necessary functions from other packages
importFrom("stats", "optim", "pbeta", "pbinom", "pnorm", "qnorm", "rbeta", "uniroot")
importFrom("stats", "pt")
importFrom("stats", "qbinom")
importFrom("stats", "ppois", "qpois")
importFrom("methods", "new")
importFrom("graphics", "abline", "grid")